/*
 * Copyright (c) Cloud Software Group, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1) Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2) Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */


using System;
using System.Collections;
using System.Collections.Generic;
using System.Management.Automation;
using XenAPI;

namespace Citrix.XenServer.Commands
{
    [Cmdlet(VerbsCommon.Get, "XenConsoleProperty", SupportsShouldProcess = false)]
    public class GetXenConsoleProperty : XenServerCmdlet
    {
        #region Cmdlet Parameters

        [Parameter(ParameterSetName = "XenObject", Mandatory = true, ValueFromPipeline = true, Position = 0)]
        public XenAPI.Console Console { get; set; }

        [Parameter(ParameterSetName = "Ref", Mandatory = true, ValueFromPipelineByPropertyName = true, Position = 0)]
        [Alias("opaque_ref")]
        public XenRef<XenAPI.Console> Ref { get; set; }


        [Parameter(Mandatory = true)]
        public XenConsoleProperty XenProperty { get; set; }

        #endregion

        #region Cmdlet Methods

        protected override void ProcessRecord()
        {
            GetSession();

            string console = ParseConsole();

            switch (XenProperty)
            {
                case XenConsoleProperty.Uuid:
                    ProcessRecordUuid(console);
                    break;
                case XenConsoleProperty.Protocol:
                    ProcessRecordProtocol(console);
                    break;
                case XenConsoleProperty.Location:
                    ProcessRecordLocation(console);
                    break;
                case XenConsoleProperty.VM:
                    ProcessRecordVM(console);
                    break;
                case XenConsoleProperty.OtherConfig:
                    ProcessRecordOtherConfig(console);
                    break;
            }

            UpdateSessions();
        }

        #endregion

        #region Private Methods

        private string ParseConsole()
        {
            string console = null;

            if (Console != null)
                console = (new XenRef<XenAPI.Console>(Console)).opaque_ref;
            else if (Ref != null)
                console = Ref.opaque_ref;
            else
            {
                ThrowTerminatingError(new ErrorRecord(
                    new ArgumentException("At least one of the parameters 'Console', 'Ref', 'Uuid' must be set"),
                    string.Empty,
                    ErrorCategory.InvalidArgument,
                    Console));
            }

            return console;
        }

        private void ProcessRecordUuid(string console)
        {
            RunApiCall(()=>
            {
                    string obj = XenAPI.Console.get_uuid(session, console);

                        WriteObject(obj, true);
            });
        }

        private void ProcessRecordProtocol(string console)
        {
            RunApiCall(()=>
            {
                    console_protocol obj = XenAPI.Console.get_protocol(session, console);

                        WriteObject(obj, true);
            });
        }

        private void ProcessRecordLocation(string console)
        {
            RunApiCall(()=>
            {
                    string obj = XenAPI.Console.get_location(session, console);

                        WriteObject(obj, true);
            });
        }

        private void ProcessRecordVM(string console)
        {
            RunApiCall(()=>
            {
                    string objRef = XenAPI.Console.get_VM(session, console);

                        XenAPI.VM obj = null;

                        if (objRef != "OpaqueRef:NULL")
                        {
                            obj = XenAPI.VM.get_record(session, objRef);
                            obj.opaque_ref = objRef;
                        }

                        WriteObject(obj, true);
            });
        }

        private void ProcessRecordOtherConfig(string console)
        {
            RunApiCall(()=>
            {
                    var dict = XenAPI.Console.get_other_config(session, console);

                        Hashtable ht = CommonCmdletFunctions.ConvertDictionaryToHashtable(dict);
                        WriteObject(ht, true);
            });
        }

        #endregion
    }

    public enum XenConsoleProperty
    {
        Uuid,
        Protocol,
        Location,
        VM,
        OtherConfig
    }

}
