/*
 * Copyright (c) Cloud Software Group, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1) Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2) Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */


using System;
using System.Collections;
using System.Collections.Generic;
using System.Management.Automation;
using XenAPI;

namespace Citrix.XenServer.Commands
{
    [Cmdlet(VerbsCommon.Get, "XenSecretProperty", SupportsShouldProcess = false)]
    public class GetXenSecretProperty : XenServerCmdlet
    {
        #region Cmdlet Parameters

        [Parameter(ParameterSetName = "XenObject", Mandatory = true, ValueFromPipeline = true, Position = 0)]
        public XenAPI.Secret Secret { get; set; }

        [Parameter(ParameterSetName = "Ref", Mandatory = true, ValueFromPipelineByPropertyName = true, Position = 0)]
        [Alias("opaque_ref")]
        public XenRef<XenAPI.Secret> Ref { get; set; }


        [Parameter(Mandatory = true)]
        public XenSecretProperty XenProperty { get; set; }

        #endregion

        #region Cmdlet Methods

        protected override void ProcessRecord()
        {
            GetSession();

            string secret = ParseSecret();

            switch (XenProperty)
            {
                case XenSecretProperty.Uuid:
                    ProcessRecordUuid(secret);
                    break;
                case XenSecretProperty.Value:
                    ProcessRecordValue(secret);
                    break;
                case XenSecretProperty.OtherConfig:
                    ProcessRecordOtherConfig(secret);
                    break;
            }

            UpdateSessions();
        }

        #endregion

        #region Private Methods

        private string ParseSecret()
        {
            string secret = null;

            if (Secret != null)
                secret = (new XenRef<XenAPI.Secret>(Secret)).opaque_ref;
            else if (Ref != null)
                secret = Ref.opaque_ref;
            else
            {
                ThrowTerminatingError(new ErrorRecord(
                    new ArgumentException("At least one of the parameters 'Secret', 'Ref', 'Uuid' must be set"),
                    string.Empty,
                    ErrorCategory.InvalidArgument,
                    Secret));
            }

            return secret;
        }

        private void ProcessRecordUuid(string secret)
        {
            RunApiCall(()=>
            {
                    string obj = XenAPI.Secret.get_uuid(session, secret);

                        WriteObject(obj, true);
            });
        }

        private void ProcessRecordValue(string secret)
        {
            RunApiCall(()=>
            {
                    string obj = XenAPI.Secret.get_value(session, secret);

                        WriteObject(obj, true);
            });
        }

        private void ProcessRecordOtherConfig(string secret)
        {
            RunApiCall(()=>
            {
                    var dict = XenAPI.Secret.get_other_config(session, secret);

                        Hashtable ht = CommonCmdletFunctions.ConvertDictionaryToHashtable(dict);
                        WriteObject(ht, true);
            });
        }

        #endregion
    }

    public enum XenSecretProperty
    {
        Uuid,
        Value,
        OtherConfig
    }

}
