/*
 * Copyright (c) Cloud Software Group, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1) Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2) Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */


using System;
using System.Collections;
using System.Collections.Generic;
using System.Management.Automation;
using XenAPI;

namespace Citrix.XenServer.Commands
{
    [Cmdlet(VerbsCommon.Get, "XenVUSBProperty", SupportsShouldProcess = false)]
    public class GetXenVUSBProperty : XenServerCmdlet
    {
        #region Cmdlet Parameters

        [Parameter(ParameterSetName = "XenObject", Mandatory = true, ValueFromPipeline = true, Position = 0)]
        public XenAPI.VUSB VUSB { get; set; }

        [Parameter(ParameterSetName = "Ref", Mandatory = true, ValueFromPipelineByPropertyName = true, Position = 0)]
        [Alias("opaque_ref")]
        public XenRef<XenAPI.VUSB> Ref { get; set; }


        [Parameter(Mandatory = true)]
        public XenVUSBProperty XenProperty { get; set; }

        #endregion

        #region Cmdlet Methods

        protected override void ProcessRecord()
        {
            GetSession();

            string vusb = ParseVUSB();

            switch (XenProperty)
            {
                case XenVUSBProperty.Uuid:
                    ProcessRecordUuid(vusb);
                    break;
                case XenVUSBProperty.AllowedOperations:
                    ProcessRecordAllowedOperations(vusb);
                    break;
                case XenVUSBProperty.CurrentOperations:
                    ProcessRecordCurrentOperations(vusb);
                    break;
                case XenVUSBProperty.VM:
                    ProcessRecordVM(vusb);
                    break;
                case XenVUSBProperty.USBGroup:
                    ProcessRecordUSBGroup(vusb);
                    break;
                case XenVUSBProperty.OtherConfig:
                    ProcessRecordOtherConfig(vusb);
                    break;
                case XenVUSBProperty.CurrentlyAttached:
                    ProcessRecordCurrentlyAttached(vusb);
                    break;
            }

            UpdateSessions();
        }

        #endregion

        #region Private Methods

        private string ParseVUSB()
        {
            string vusb = null;

            if (VUSB != null)
                vusb = (new XenRef<XenAPI.VUSB>(VUSB)).opaque_ref;
            else if (Ref != null)
                vusb = Ref.opaque_ref;
            else
            {
                ThrowTerminatingError(new ErrorRecord(
                    new ArgumentException("At least one of the parameters 'VUSB', 'Ref', 'Uuid' must be set"),
                    string.Empty,
                    ErrorCategory.InvalidArgument,
                    VUSB));
            }

            return vusb;
        }

        private void ProcessRecordUuid(string vusb)
        {
            RunApiCall(()=>
            {
                    string obj = XenAPI.VUSB.get_uuid(session, vusb);

                        WriteObject(obj, true);
            });
        }

        private void ProcessRecordAllowedOperations(string vusb)
        {
            RunApiCall(()=>
            {
                    List<vusb_operations> obj = XenAPI.VUSB.get_allowed_operations(session, vusb);

                        WriteObject(obj, true);
            });
        }

        private void ProcessRecordCurrentOperations(string vusb)
        {
            RunApiCall(()=>
            {
                    var dict = XenAPI.VUSB.get_current_operations(session, vusb);

                        Hashtable ht = CommonCmdletFunctions.ConvertDictionaryToHashtable(dict);
                        WriteObject(ht, true);
            });
        }

        private void ProcessRecordVM(string vusb)
        {
            RunApiCall(()=>
            {
                    string objRef = XenAPI.VUSB.get_VM(session, vusb);

                        XenAPI.VM obj = null;

                        if (objRef != "OpaqueRef:NULL")
                        {
                            obj = XenAPI.VM.get_record(session, objRef);
                            obj.opaque_ref = objRef;
                        }

                        WriteObject(obj, true);
            });
        }

        private void ProcessRecordUSBGroup(string vusb)
        {
            RunApiCall(()=>
            {
                    string objRef = XenAPI.VUSB.get_USB_group(session, vusb);

                        XenAPI.USB_group obj = null;

                        if (objRef != "OpaqueRef:NULL")
                        {
                            obj = XenAPI.USB_group.get_record(session, objRef);
                            obj.opaque_ref = objRef;
                        }

                        WriteObject(obj, true);
            });
        }

        private void ProcessRecordOtherConfig(string vusb)
        {
            RunApiCall(()=>
            {
                    var dict = XenAPI.VUSB.get_other_config(session, vusb);

                        Hashtable ht = CommonCmdletFunctions.ConvertDictionaryToHashtable(dict);
                        WriteObject(ht, true);
            });
        }

        private void ProcessRecordCurrentlyAttached(string vusb)
        {
            RunApiCall(()=>
            {
                    bool obj = XenAPI.VUSB.get_currently_attached(session, vusb);

                        WriteObject(obj, true);
            });
        }

        #endregion
    }

    public enum XenVUSBProperty
    {
        Uuid,
        AllowedOperations,
        CurrentOperations,
        VM,
        USBGroup,
        OtherConfig,
        CurrentlyAttached
    }

}
