/*
 * Copyright (c) Cloud Software Group, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1) Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2) Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */


using System;
using System.Collections;
using System.Collections.Generic;
using System.Management.Automation;
using XenAPI;

namespace Citrix.XenServer.Commands
{
    [Cmdlet(VerbsCommon.Set, "XenVMSS", SupportsShouldProcess = true)]
    [OutputType(typeof(XenAPI.VMSS))]
    [OutputType(typeof(void))]
    public class SetXenVMSS : XenServerCmdlet
    {
        #region Cmdlet Parameters

        [Parameter]
        public SwitchParameter PassThru { get; set; }

        [Parameter(ParameterSetName = "XenObject", Mandatory = true, ValueFromPipeline = true, Position = 0)]
        public XenAPI.VMSS VMSS { get; set; }

        [Parameter(ParameterSetName = "Ref", Mandatory = true, ValueFromPipelineByPropertyName = true, Position = 0)]
        [Alias("opaque_ref")]
        public XenRef<XenAPI.VMSS> Ref { get; set; }

        [Parameter(ParameterSetName = "Uuid", Mandatory = true, ValueFromPipelineByPropertyName = true, Position = 0)]
        public Guid Uuid { get; set; }

        [Parameter(ParameterSetName = "Name", Mandatory = true, ValueFromPipelineByPropertyName = true, Position = 0)]
        [Alias("name_label")]
        public string Name { get; set; }


        [Parameter]
        public string NameLabel
        {
            get { return _nameLabel; }
            set
            {
                _nameLabel = value;
                _nameLabelIsSpecified = true;
            }
        }
        private string _nameLabel;
        private bool _nameLabelIsSpecified;

        [Parameter]
        public string NameDescription
        {
            get { return _nameDescription; }
            set
            {
                _nameDescription = value;
                _nameDescriptionIsSpecified = true;
            }
        }
        private string _nameDescription;
        private bool _nameDescriptionIsSpecified;

        [Parameter]
        public bool Enabled
        {
            get { return _enabled; }
            set
            {
                _enabled = value;
                _enabledIsSpecified = true;
            }
        }
        private bool _enabled;
        private bool _enabledIsSpecified;

        [Parameter]
        public long RetainedSnapshots
        {
            get { return _retainedSnapshots; }
            set
            {
                _retainedSnapshots = value;
                _retainedSnapshotsIsSpecified = true;
            }
        }
        private long _retainedSnapshots;
        private bool _retainedSnapshotsIsSpecified;

        [Parameter]
        public vmss_frequency Frequency
        {
            get { return _frequency; }
            set
            {
                _frequency = value;
                _frequencyIsSpecified = true;
            }
        }
        private vmss_frequency _frequency;
        private bool _frequencyIsSpecified;

        [Parameter]
        public Hashtable Schedule
        {
            get { return _schedule; }
            set
            {
                _schedule = value;
                _scheduleIsSpecified = true;
            }
        }
        private Hashtable _schedule;
        private bool _scheduleIsSpecified;

        [Parameter]
        public DateTime LastRunTime
        {
            get { return _lastRunTime; }
            set
            {
                _lastRunTime = value;
                _lastRunTimeIsSpecified = true;
            }
        }
        private DateTime _lastRunTime;
        private bool _lastRunTimeIsSpecified;

        [Parameter]
        public vmss_type Type
        {
            get { return _type; }
            set
            {
                _type = value;
                _typeIsSpecified = true;
            }
        }
        private vmss_type _type;
        private bool _typeIsSpecified;

        #endregion

        #region Cmdlet Methods

        protected override void ProcessRecord()
        {
            GetSession();

            string vmss = ParseVMSS();

            if (_nameLabelIsSpecified)
                ProcessRecordNameLabel(vmss);
            if (_nameDescriptionIsSpecified)
                ProcessRecordNameDescription(vmss);
            if (_enabledIsSpecified)
                ProcessRecordEnabled(vmss);
            if (_retainedSnapshotsIsSpecified)
                ProcessRecordRetainedSnapshots(vmss);
            if (_frequencyIsSpecified)
                ProcessRecordFrequency(vmss);
            if (_scheduleIsSpecified)
                ProcessRecordSchedule(vmss);
            if (_lastRunTimeIsSpecified)
                ProcessRecordLastRunTime(vmss);
            if (_typeIsSpecified)
                ProcessRecordType(vmss);

            if (!PassThru)
                return;

            RunApiCall(() =>
                {
                    var contxt = _context as XenServerCmdletDynamicParameters;

                    if (contxt != null && contxt.Async)
                    {
                        XenAPI.Task taskObj = null;
                        if (taskRef != null && taskRef != "OpaqueRef:NULL")
                        {
                            taskObj = XenAPI.Task.get_record(session, taskRef.opaque_ref);
                            taskObj.opaque_ref = taskRef.opaque_ref;
                        }

                        WriteObject(taskObj, true);
                    }
                    else
                    {

                        var obj = XenAPI.VMSS.get_record(session, vmss);
                        if (obj != null)
                            obj.opaque_ref = vmss;
                        WriteObject(obj, true);

                    }
                });

            UpdateSessions();
        }

        #endregion

        #region Private Methods

        private string ParseVMSS()
        {
            string vmss = null;

            if (VMSS != null)
                vmss = (new XenRef<XenAPI.VMSS>(VMSS)).opaque_ref;
            else if (Uuid != Guid.Empty)
            {
                var xenRef = XenAPI.VMSS.get_by_uuid(session, Uuid.ToString());
                if (xenRef != null)
                    vmss = xenRef.opaque_ref;
            }
            else if (Name != null)
            {
                var xenRefs = XenAPI.VMSS.get_by_name_label(session, Name);
                if (xenRefs.Count == 1)
                    vmss = xenRefs[0].opaque_ref;
                else if (xenRefs.Count > 1)
                    ThrowTerminatingError(new ErrorRecord(
                        new ArgumentException(string.Format("More than one XenAPI.VMSS with name label {0} exist", Name)),
                        string.Empty,
                        ErrorCategory.InvalidArgument,
                        Name));
            }
            else if (Ref != null)
                vmss = Ref.opaque_ref;
            else
            {
                ThrowTerminatingError(new ErrorRecord(
                    new ArgumentException("At least one of the parameters 'VMSS', 'Ref', 'Uuid' must be set"),
                    string.Empty,
                    ErrorCategory.InvalidArgument,
                    VMSS));
            }

            return vmss;
        }

        private void ProcessRecordNameLabel(string vmss)
        {
            if (!ShouldProcess(vmss, "VMSS.set_name_label"))
                return;

            RunApiCall(()=>
            {
                    XenAPI.VMSS.set_name_label(session, vmss, NameLabel);

            });
        }

        private void ProcessRecordNameDescription(string vmss)
        {
            if (!ShouldProcess(vmss, "VMSS.set_name_description"))
                return;

            RunApiCall(()=>
            {
                    XenAPI.VMSS.set_name_description(session, vmss, NameDescription);

            });
        }

        private void ProcessRecordEnabled(string vmss)
        {
            if (!ShouldProcess(vmss, "VMSS.set_enabled"))
                return;

            RunApiCall(()=>
            {
                    XenAPI.VMSS.set_enabled(session, vmss, Enabled);

            });
        }

        private void ProcessRecordRetainedSnapshots(string vmss)
        {
            if (!ShouldProcess(vmss, "VMSS.set_retained_snapshots"))
                return;

            RunApiCall(()=>
            {
                    XenAPI.VMSS.set_retained_snapshots(session, vmss, RetainedSnapshots);

            });
        }

        private void ProcessRecordFrequency(string vmss)
        {
            if (!ShouldProcess(vmss, "VMSS.set_frequency"))
                return;

            RunApiCall(()=>
            {
                    XenAPI.VMSS.set_frequency(session, vmss, Frequency);

            });
        }

        private void ProcessRecordSchedule(string vmss)
        {
            if (!ShouldProcess(vmss, "VMSS.set_schedule"))
                return;

            RunApiCall(()=>
            {
                    XenAPI.VMSS.set_schedule(session, vmss, CommonCmdletFunctions.ConvertHashTableToDictionary<string, string>(Schedule));

            });
        }

        private void ProcessRecordLastRunTime(string vmss)
        {
            if (!ShouldProcess(vmss, "VMSS.set_last_run_time"))
                return;

            RunApiCall(()=>
            {
                    XenAPI.VMSS.set_last_run_time(session, vmss, LastRunTime);

            });
        }

        private void ProcessRecordType(string vmss)
        {
            if (!ShouldProcess(vmss, "VMSS.set_type"))
                return;

            RunApiCall(()=>
            {
                    XenAPI.VMSS.set_type(session, vmss, Type);

            });
        }

        #endregion
    }
}
